/**
 * Makes an input field to a user selection field
 *
 * @author www.pcsg.de (Henning Leutz)
 *
 * @module controls/users/Input
 * @package com.pcsg.qui.js.controls.users
 * @namespace QUI.controls.users
 *
 * @require controls/Control
 * @require controls/buttons/Button
 * @require controls/users/Entry
 *
 * @event onAdd [this, userid]
 */

define('controls/users/Input', [

    'controls/Control',
    'controls/buttons/Button',
    'controls/users/Entry',

    'css!controls/users/Input.css'

], function(QUI_Control)
{
    "use strict";

    QUI.namespace( 'controls.users' );

    /**
     * @class QUI.controls.users.Input
     *
     * @param {Object} options
     * @param {DOMNode Input} Input [optional] -> if no input given, one would be created
     *
     * @memberof! <global>
     */
    QUI.controls.users.Input = new Class({

        Implements : [ QUI_Control ],
        Type       : 'QUI.controls.users.Input',

        Binds : [
            'close',
            'fireSearch',
            'update'
        ],

        options : {
            max      : false,
            multible : true,
            name     : '',
            styles   : false
        },

        initialize : function(options, Input)
        {
            this.init( options );

            this.$search = false;

            this.$Input     = null;
            this.$Elm       = false;
            this.$Container = null;
            this.$search    = false;
            this.$DropDown  = null;

            this.$Parent = Input || null;
        },

        /**
         * Return the DOMNode of the users search
         *
         * @return {DOMNode}
         */
        create : function()
        {
            this.$Elm = new Element( 'div.users-input' );

            if ( !this.$Parent )
            {
                this.$Parent = new Element('input', {
                    name : this.getAttribute('name')
                }).inject( this.$Elm );
            } else
            {
                this.$Elm.wraps( this.$Parent );
            }

            if ( this.getAttribute( 'styles' ) ) {
                this.$Elm.setStyles( this.getAttribute( 'styles' ) );
            }


            this.$Parent.set('type', 'hidden' );

            this.$Input = new Element('input', {
                type   : 'text',
                name   : this.$Parent.get('name') +'-search',
                styles : {
                    'float'       : 'left',
                    'margin'      : '3px 0',
                    'paddingLeft' : 20,
                    'background'  : 'url('+ URL_BIN_DIR +'10x10/search.png) no-repeat 4px center',
                    width         : 165,
                    cursor        : 'pointer'
                },
                events :
                {
                    keyup : function(event)
                    {
                        if ( event.key === 'down' )
                        {
                            this.down();
                            return;
                        }

                        if ( event.key === 'up' )
                        {
                            this.up();
                            return;
                        }

                        if ( event.key === 'enter' )
                        {
                            this.submit();
                            return;
                        }

                        this.fireSearch();
                    }.bind( this ),

                    blur  : this.close,
                    focus : this.fireSearch
                }
            }).inject( this.$Parent, 'before' );


            this.$DropDown = new Element('div.users-input-dropdown', {
                styles : {
                    display : 'none',
                    top  : this.$Input.getPosition().y + this.$Input.getSize().y,
                    left : this.$Input.getPosition().x
                }
            }).inject( document.body );

            this.$Container = new Element('div', {
                styles : {
                    'float' : 'left',
                    margin  : '0 0 0 10px',
                    width   : 400
                }
            }).inject( this.$Input, 'after' );

            // loading
            if ( this.$Parent.value === '' ) {
                return this.$Elm;
            }

            var i, len;
            var values = this.$Parent.value.toString().split(',');

            for ( i = 0, len = values.length; i < len; i++ )
            {
                if ( values[i] !== '' ) {
                    this.addUser( values[i] );
                }
            }

            return this.$Elm;
        },

        /**
         * updates the users search field
         */
        update : function()
        {
            if ( !this.$Container ) {
                return;
            }

            // set value
            var i, len;

            var list = this.$Container.getElements('.users-entry'),
                ids  = [];

            for ( i = 0, len = list.length; i < len; i++ ) {
                ids.push( list[i].get( 'data-id' ) );
            }

            this.$Parent.set(
                'value',
                ','+ ids.join(',') +','
            );
        },

        /**
         * fire the search
         */
        fireSearch : function()
        {
            this.cancelSearch();

            this.$DropDown.set({
                html   : '<img src="'+ URL_BIN_DIR +'images/loader.gif" />',
                styles : {
                    display : '',
                    top     : this.$Input.getPosition().y + this.$Input.getSize().y,
                    left    : this.$Input.getPosition().x
                }
            });

            this.$search = this.search.delay( 500, this );
        },

        /**
         * cancel the search timeout
         */
        cancelSearch : function()
        {
            if ( this.$search ) {
                clearTimeout( this.$search );
            }
        },

        /**
         * close the users search
         */
        close : function()
        {
            this.cancelSearch();
            this.$DropDown.setStyle( 'display', 'none' );
            this.$Input.value = '';
        },

        /**
         * Add a users to the field
         *
         * @param {Integer} uid - User-ID
         */
        addUser : function(uid)
        {
            if ( !uid ) {
                return;
            }

            if ( this.$Container.getElement( '.user-entry[data-id="'+ uid +'"]') ) {
                return;
            }

            var entries = this.$Container.getElements( '.user-entry' );

            if ( this.getAttribute( 'max' ) &&
                 this.getAttribute( 'max' ) <= entries.length )
            {
                return;
            }

            new QUI.controls.users.Entry(uid, {
                events : {
                    onDestroy : this.update
                }
            }).inject( this.$Container );

            this.fireEvent( 'add', [ this, uid ] );
            this.update();
        },

        /**
         * trigger a users search and open a user dropdown for selection
         */
        search : function()
        {
            QUI.Ajax.get('ajax_users_search', function(result, Request)
            {
                var i, len, nam, func_mousedown, func_mouseover;

                var data     = result.data,
                    value    = Request.getAttribute( 'value' ),
                    Elm      = Request.getAttribute( 'Elm' ),
                    DropDown = Elm.$DropDown;

                DropDown.set( 'html', '' );

                if ( !data.length )
                {
                    new Element('div', {
                        html   : 'Keine Ergebnisse gefunden',
                        styles : {
                            'float' : 'left',
                            'clear' : 'both',
                            padding : 5,
                            margin  : 5
                        }
                    }).inject( DropDown );

                    return;
                }

                // events
                func_mousedown = function(event)
                {
                    this.addUser(
                        event.target.get( 'data-id' )
                    );

                }.bind( Elm );

                func_mouseover = function()
                {
                    this.getParent().getElements( '.hover' ).removeClass( 'hover' );
                    this.addClass( 'hover' );
                };

                // create
                for ( i = 0, len = data.length; i < len; i++ )
                {
                    nam = data[ i ].username.toString().replace(
                        new RegExp('('+ value +')', 'gi'),
                        '<span class="mark">$1</span>'
                    );

                    new Element('div', {
                        html        : nam +' ('+ data[ i ].id +')',
                        'class'     : 'box-sizing radius5',
                        'data-id'   : data[ i ].id,
                        'data-name' : data[ i ].username,
                        styles : {
                            'float' : 'left',
                            'clear' : 'both',
                            padding : 5,
                            cursor  : 'pointer',
                            width   : '100%'
                        },
                        events :
                        {
                            mousedown : func_mousedown,
                            mouseover : func_mouseover
                        }
                    }).inject( DropDown );
                }
            }, {
                Elm    : this,
                value  : this.$Input.value,
                params : JSON.encode({
                    order  : 'ASC',
                    limit  : 5,
                    page   : 1,
                    search : true,
                    searchSettings : {
                        userSearchString : this.$Input.value
                    }
                })
            });
        },

        /**
         * keyup - users dropdown selection one step up
         *
         * @return {this}
         */
        up : function()
        {
            if ( !this.$DropDown ) {
                return this;
            }

            var Active = this.$DropDown.getElement( '.hover' );

            // Last Element
            if ( !Active )
            {
                this.$DropDown.getLast().addClass( 'hover' );
                return this;
            }

            Active.removeClass( 'hover' );

            if ( !Active.getPrevious() )
            {
                this.up();
                return this;
            }

            Active.getPrevious().addClass( 'hover' );
        },

        /**
         * keydown - users dropdown selection one step down
         *
         * @return {this}
         */
        down : function()
        {
            if ( !this.$DropDown ) {
                return this;
            }

            var Active = this.$DropDown.getElement( '.hover' );

            // First Element
            if ( !Active )
            {
                this.$DropDown.getFirst().addClass( 'hover' );
                return this;
            }

            Active.removeClass( 'hover' );

            if ( !Active.getNext() )
            {
                this.down();
                return this;
            }

            Active.getNext().addClass( 'hover' );

            return this;
        },

        /**
         * select the selected users
         */
        submit : function()
        {
            if ( !this.$DropDown ) {
                return;
            }

            var Active = this.$DropDown.getElement( '.hover' );

            if ( Active ) {
                this.addUser( Active.get( 'data-id' ) );
            }

            this.$Input.value = '';
            this.search();
        },

        /**
         * Set the focus to the input field
         *
         * @return {this}
         */
        focus : function()
        {
            if ( this.$Input ) {
                this.$Input.focus();
            }

            return this;
        }
    });

    return QUI.controls.users.Input;
});